﻿#region

using System.IO;
using System.Reflection;
using System.Text;
using NUnit.Framework;

#endregion


namespace OMS.Ice.T4Generator.Test
{
    public abstract class TestCase
    {
        private IGenerator _generator;

        protected IGenerator Generator
        {
            get { return _generator; }
        }

        [SetUp]
        public void SetUp()
        {
            _generator = new Generator();
        }


        [TearDown]
        public void TearDown()
        {
            _generator = null;
        }


        protected void Run( string template, string expectedResult, params object[] data )
        {
            var generator = Generator;

            //generator.Settings.Options.Add( "CompilerVersion", "v4.0" );
            //generator.Settings.EndOfLine = EndOfLine.CRLF;
            //generator.Settings.CompilerVersion = CompilerVersion.Template;

            var stream = new MemoryStream();
            var textWriter = new StreamWriter( stream, Encoding.UTF32 );

            generator.Generate( BuildPath( template ), textWriter, data );

            stream.Seek( 0, SeekOrigin.Begin );
            var textReader = new StreamReader( stream );

            var generatedText = textReader.ReadToEnd();

            Assert.AreEqual( expectedResult, generatedText );
        }

        protected string Run( string template, object[] data )
        {
            var generator = Generator;

            var stream = new MemoryStream();
            var textWriter = new StreamWriter( stream, Encoding.UTF32 );

            generator.Generate( BuildPath( template ), textWriter, data );

            stream.Seek( 0, SeekOrigin.Begin );
            var textReader = new StreamReader( stream );

            return textReader.ReadToEnd();
        }


        protected string BuildPath( string template )
        {
            if (Path.IsPathRooted(template))
                return template;

            var path = Path.GetDirectoryName( Assembly.GetExecutingAssembly().Location );
            return Path.Combine( path, template );
        }
    }
}