﻿#region

using System.Collections.Generic;
using System.Text;
using OMS.Ice.T4Generator.Properties;
using OMS.Ice.T4Generator.Syntax;

#endregion


namespace OMS.Ice.T4Generator.CodeBuilder
{
    internal class VBCodeBuilder : CodeBuilder
    {
        protected override string GetCodeTemplate()
        {
            return Resources.VBCodeTemplate;
        }

        protected override string GetIncludeTemplate()
        {
            return Resources.VBIncludeTemplate;
        }

        protected override StringBuilder CreateImports( IEnumerable<string> imports )
        {
            var result = new StringBuilder();

            foreach( var import in imports )
                result.AppendLine( string.Format( "Imports {0}", import ) );

            return result;
        }

        protected override StringBuilder CreateFields( IEnumerable<Parameter> parameters )
        {
            var result = new StringBuilder();

            foreach( var parameter in parameters )
                result.AppendLine( string.Format( "\t\tPrivate {0} as {1}", parameter.Name, parameter.Type ) );

            return result;
        }

        protected override StringBuilder CreateParameters( IEnumerable<Parameter> parameters )
        {
            var result = new StringBuilder();

            var first = true;
            foreach( var parameter in parameters )
            {
                if( !first )
                    result.Append( ", " );
                first = false;

                result.Append( string.Format( "ByVal {0} As {1}", parameter.Name, parameter.Type ) );
            }

            return result;
        }

        protected override StringBuilder CreateCallParameters( IEnumerable<Parameter> parameters )
        {
            var result = new StringBuilder();

            var first = true;
            foreach( var parameter in parameters )
            {
                if( !first )
                    result.Append( ", " );
                first = false;

                result.Append( string.Format( "{0}", parameter.Name ) );
            }

            return result;
        }

        protected override StringBuilder CreateFieldInitializers( IEnumerable<Parameter> parameters )
        {
            var result = new StringBuilder();

            foreach( var parameter in parameters )
                result.AppendLine( string.Format( "\t\t\tMe.{0} = {0}", parameter.Name ) );

            return result;
        }

        protected override void CreateTextBlock( StringBuilder result, IEnumerable<LineInfo> content )
        {
            foreach( var line in content )
                result.AppendLine( string.Format( "\t\t\tWrite( \"{0}\" {1} )", Escape( line.Text ), line.HasEol ? GetEndOfLine() : string.Empty ) );
        }

        protected override void CreateInlineCode( StringBuilder result, string statement )
        {
            result.AppendLine( string.Format( "\t\t\tWrite( {0} )", statement ) );
        }

        protected override StringBuilder CreateMethodCall( StringBuilder result, IncludeDirective directive )
        {
            var parameters = GetParameters( IncludedTemplates[directive] );

            result.AppendLine( string.Format( "\t\t\t{0}TemplateMethod({1})", directive.Name, CreateCallParameters( parameters ) ) );

            return result;
        }

        protected override string BeginLinePragma( Part part )
        {
            return string.Format( "#ExternalSource(\"{0}\", {1})\r\n", part.Source, part.Line );
        }

        protected override string EndLinePragma()
        {
            return string.Format( "#End ExternalSource\r\n" );
        }


        private static string Escape( string text )
        {
            return text.Replace( "\"", "\"\"" );
        }

        protected override string GetEndOfLine()
        {
            switch( EndOfLine )
            {
                case EndOfLine.CR:
                    return "& Microsoft.VisualBasic.Constants.vbCr";
                case EndOfLine.LFCR:
                    return "& Microsoft.VisualBasic.Constants.vbLf & Microsoft.VisualBasic.Constants.vbCr";
                case EndOfLine.CRLF:
                    return "& Microsoft.VisualBasic.Constants.vbCrLf";
                case EndOfLine.LF:
                    return "& Microsoft.VisualBasic.Constants.vbLf";
                case EndOfLine.NEL:
                    return "& Microsoft.VisualBasic.Strings.ChrW(&H0085)";
                case EndOfLine.LS:
                    return "& Microsoft.VisualBasic.Strings.ChrW(&H2028)";
                case EndOfLine.PS:
                    return "& Microsoft.VisualBasic.Strings.ChrW(&H2029)";
                default:
                    return "& Microsoft.VisualBasic.Constants.vbCrLf";
            }
        }

        protected override string EscapeIdentifier( string possibleIdentifier )
        {
            return possibleIdentifier.StartsWith( "[" ) ? possibleIdentifier : "[" + possibleIdentifier + "]";
        }
    }
}